"use client";
import { useMemo } from "react";
import Pagination from "./Pagination";
import classNames from "classnames";
import ColHeaderCell from "./ColHeaderCell";
var getTypeComparator = function (typeStr) {
    if (typeStr === "date" || typeStr === "datetime") {
        return function (a, b) { return a.valueOf() - b.valueOf(); };
    }
    if (typeStr === "number") {
        return function (a, b) { return a - b; };
    }
    return function (a, b) {
        if (a < b) {
            return -1;
        }
        if (a > b) {
            return 1;
        }
        return 0;
    };
};
var getRowComparator = function (comparator, fieldName) {
    return function (rowA, rowB) { return comparator(rowA[fieldName], rowB[fieldName]); };
};
var Grid = function (_a) {
    var rows = _a.rows, cols = _a.cols, pagination = _a.pagination, sortModel = _a.sortModel;
    var sortedRows = useMemo(function () {
        if (!sortModel || !sortModel.sortColDef) {
            return rows;
        }
        var sortFieldName = sortModel.sortColDef.name;
        var sortOrder = sortModel.sortColDef.order;
        var sortColIndex = cols.findIndex(function (_a) {
            var name = _a.name;
            return name === sortFieldName;
        });
        if (sortColIndex < 0) {
            throw new Error("The sortModel for the grid specifies that the data should be sorted based on a column named ".concat(sortFieldName, ", but it was not found among the column definitions."));
        }
        var typeStr = cols[sortColIndex].type;
        var ascComparator = getTypeComparator(typeStr);
        var rowComparator = getRowComparator(ascComparator, sortFieldName);
        if (sortOrder === "desc") {
            var descComparator = function (a, b) {
                return ascComparator(a, b) * -1;
            };
            rowComparator = getRowComparator(descComparator, sortFieldName);
        }
        return rows.slice().sort(rowComparator);
    }, [rows, cols, sortModel]);
    var currentPageRows = useMemo(function () {
        if (pagination === undefined) {
            return sortedRows;
        }
        var pageSizeOptions = pagination.pageSizeOptions, pageSizeIndex = pagination.pageSizeIndex, currentPage = pagination.currentPage;
        var pageSize = pageSizeOptions[pageSizeIndex];
        var lowerIndex = pageSize * (currentPage - 1);
        var upperIndex = lowerIndex + pageSize;
        return sortedRows.slice(lowerIndex, upperIndex);
    }, [sortedRows, pagination]);
    var displayRows = useMemo(function () {
        var nameToIndex = new Map();
        cols.forEach(function (_a, index) {
            var name = _a.name;
            nameToIndex.set(name, index);
        });
        return currentPageRows.map(function (row, index) {
            cols
                .map(function (_a) {
                var name = _a.name;
                return name;
            })
                .forEach(function (name) {
                if (!(name in row)) {
                    throw new Error("Column definition specifies a property named \"".concat(name, "\", but it was not found in the row data object at index ").concat(index, "."));
                }
            });
            var displayRow = [];
            Object.keys(row).forEach(function (name) {
                if (!nameToIndex.has(name)) {
                    throw new Error("Row data contains a property named \"".concat(name, "\", but it was not found among the column definitions."));
                }
                var index = nameToIndex.get(name);
                var formatter = cols[index].formatter;
                var typeString = cols[index].type;
                var value = row[name];
                if (formatter) {
                    displayRow[index] = formatter(value);
                    return;
                }
                if (typeString === "date") {
                    displayRow[index] = value.toDateString();
                    return;
                }
                if (typeString === "datetime") {
                    displayRow[index] = value.toLocaleString();
                    return;
                }
                if (typeString === "number") {
                    displayRow[index] = value.toLocaleString();
                    return;
                }
                displayRow[index] = value;
            });
            return displayRow;
        });
    }, [currentPageRows, cols]);
    var handleSetPageNum = function (pageNum) {
        if (pagination === undefined) {
            return;
        }
        pagination.setCurrentPage(pageNum);
    };
    var handleSetPageSize = function (event) {
        if (pagination === undefined) {
            return;
        }
        pagination.setPageSizeIndex(Number(event.target.value));
    };
    // Once this component implements selection state, and if such interactivity is enabled, (conditionally) change the
    // aria role to "grid".
    // Array index is okay for the key for rows until some type of feature involving changing the index of rows, such as
    // sorting or pagination, is implemented.
    return (<div>
      <table className="table">
        <thead>
          <tr>
            {cols.map(function (_a) {
            var _b;
            var name = _a.name, label = _a.label, sortable = _a.sortable;
            var colSortModel = sortModel && sortable
                ? {
                    sortOrder: ((_b = sortModel.sortColDef) === null || _b === void 0 ? void 0 : _b.name) === name
                        ? sortModel.sortColDef.order
                        : null,
                    setSortOrder: function (order) {
                        sortModel.setSortColDef(order && { name: name, order: order });
                    },
                }
                : undefined;
            return (<ColHeaderCell key={name} label={label} sortModel={colSortModel}/>);
        })}
          </tr>
        </thead>
        <tbody>
          {displayRows.map(function (row, index) { return (<tr key={index}>
              {row.map(function (value, index) { return (<td key={index}>{value}</td>); })}
            </tr>); })}
        </tbody>
      </table>
      {pagination && (<div className="d-flex justify-content-end gap-2">
          <div>
            <select className={classNames({
                "form-select": true,
                "form-select-lg": pagination.componentSize === "large",
                "form-select-sm": pagination.componentSize === "small",
            })} value={pagination.pageSizeIndex} aria-label="Number of Rows per Page" onChange={handleSetPageSize}>
              {pagination.pageSizeOptions.map(function (numRows, index) { return (<option key={index} value={index}>
                  {numRows}
                </option>); })}
            </select>
          </div>
          <Pagination numPages={Math.ceil(rows.length /
                pagination.pageSizeOptions[pagination.pageSizeIndex])} pageNum={pagination.currentPage} numButtons={pagination.maxPageButtons} setPageNum={handleSetPageNum} size={pagination.componentSize || "medium"}/>
        </div>)}
    </div>);
};
export default Grid;
